package com.yandex.syrinxsdk.simplesample;

import android.os.Bundle;
import android.os.Handler;
import android.os.HandlerThread;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

import com.yandex.syrinxsdk.Syrinx;

public class Sample2Activity extends AppCompatActivity {
    //UI
    private EditText messageEt;
    private Button transmitBtn;
    private TextView stateView;

    //library object instance to transmit and receive messages
    private Syrinx syrinx;

    //bkg thread to execute long-term operations
    private HandlerThread bkgHandlerThread = new HandlerThread("BackgroundThread");
    //a handler to post operations to bkg thread
    private Handler bkgThreadHandler;
    // a handler to post messages to UI
    private Handler uiHandler;

    @Override
    protected void onResume() {
        super.onResume();
        //don't forget to start the library
        //this step is required to start native stuff
        syrinx.startLibrary();
        //wait for a message
        syrinx.startListening();
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        //initialize the Syrinx object using context
        syrinx = new Syrinx(getApplicationContext());
        //initialize bkg handler with the looper of the bkg thread
        bkgHandlerThread.start();
        bkgThreadHandler = new Handler(bkgHandlerThread.getLooper());
        //UI as usual
        setUI();
        //ui handler
        uiHandler = new Handler();
        //logic
        work();
    }

    private void setUI() {
        setContentView(R.layout.activity_main);
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        messageEt = (EditText) findViewById(R.id.message_et);
        messageEt.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
                //do nothing
            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                //do nothing
            }

            @Override
            public void afterTextChanged(Editable s) {
                enableTransmitBtn();
            }
        });
        transmitBtn = (Button) findViewById(R.id.transmit_btn);
        transmitBtn.setEnabled(false);
        transmitBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                transmitMessage(messageEt.getText().toString());
            }
        });

        stateView = (TextView) findViewById(R.id.state_tv);
    }


    private void enableTransmitBtn() {
        transmitBtn.setEnabled(messageEt.getText().length() > 0 && !syrinx.isReceiving() && !syrinx.isTransmitting());
    }

    //send a message
    private void transmitMessage(final String message) {
        bkgThreadHandler.post(new Runnable() {
            @Override
            public void run() {
                //do not listen while transmitting
                syrinx.stopListening();
                uiHandler.post(new Runnable() {
                    @Override
                    public void run() {
                        stateView.setText(R.string.state_transmitting_started);
                        transmitBtn.setEnabled(false);
                    }
                });
                //the method is sync, so use a non-UI thread to send a message
                syrinx.transmit(message);
                uiHandler.post(new Runnable() {
                    @Override
                    public void run() {
                        stateView.setText(R.string.state_transmitting_ended);
                        enableTransmitBtn();
                    }
                });
                //ready to listen again
                syrinx.startListening();
            }
        });
    }

    private void work() {
        bkgThreadHandler.post(new Runnable() {
            private boolean isReceiving;

            @Override
            public void run() {
                String messageToPost = "";
                if (syrinx.isReceiving() && !isReceiving) {
                    isReceiving = true;
                    messageToPost = getString(R.string.state_receiving_started);
                }
                if (!syrinx.isReceiving() && isReceiving) {
                    isReceiving = false;
                    messageToPost = getString(R.string.state_receiving_ended);
                }
                if (syrinx.isReceiving()) {
                    messageToPost = getString(R.string.state_message_partial_result, syrinx.peekUnfinishedMessage());
                }
                if (!syrinx.isReceiving() && !syrinx.hasAnyMessages()) {
                    String word = syrinx.getReceivedMessage();
                    messageToPost = getString(R.string.state_message_received, word);
                }
                final String finalMessageToPost = messageToPost;
                uiHandler.post(new Runnable() {
                    @Override
                    public void run() {
                        enableTransmitBtn();
                        stateView.setText(finalMessageToPost);
                    }
                });
                bkgThreadHandler.postDelayed(this, 500);
            }

        });
    }

    @Override
    protected void onPause() {
        super.onPause();
        syrinx.stopListening();
        syrinx.stopTransmitting();
        syrinx.stopLibrary();
    }
}
