package com.yandex.syrinxsdk.sample.syrinx;

import android.app.Service;
import android.content.Intent;
import android.os.Handler;
import android.os.IBinder;

import com.yandex.syrinxsdk.Syrinx;

import java.util.Arrays;

public class SyrinxService extends Service {

    public static final int MAX_LEVEL = 120;
    public static final int MIN_LEVEL = 0;
    public static final int MAX_BYTES_TO_TRANSMIT = 255;

    private volatile SyrinxListener chatterCallback;
    private Syrinx syrinx;
    private SyrinxBinder binder;

    private volatile boolean isTransmittingCancelled;

    private Handler handler = new Handler();

    @Override
    public void onCreate() {
        syrinx = new Syrinx(getApplicationContext());
        binder = new SyrinxBinder(this);
        syrinx.startLibrary();
    }


    @Override
    public int onStartCommand(final Intent intent, int flags, int startId) {
        final Runnable runnable = new Runnable() {
            private boolean stateReceiving;

            @Override
            public void run() {
                if (chatterCallback != null) {
                    chatterCallback.onLevelChanged((int) (syrinx.getLevel() + MAX_LEVEL), MIN_LEVEL, MAX_LEVEL);
                    if (syrinx.isReceiving() && !stateReceiving) {
                        stateReceiving = true;
                        chatterCallback.onReceivingStarted();
                    }
                    if (!syrinx.isReceiving() && stateReceiving) {
                        stateReceiving = false;
                        chatterCallback.onReceivingEnded();
                    }
                    if (syrinx.isReceiving()) {
                        chatterCallback.onMessagePartReceived(syrinx.peekUnfinishedMessage());
                    }
                    if (!syrinx.isReceiving() && !syrinx.hasAnyMessages()) {
                        String word = syrinx.getReceivedMessage();
                        chatterCallback.onMessageReceived(word);
                    }
                }
                handler.postDelayed(this, 100);
            }
        };
        handler.post(runnable);
        return START_STICKY;
    }


    @Override
    public IBinder onBind(Intent intent) {
        return binder;
    }


    @Override
    public boolean onUnbind(Intent intent) {
        binder.unsubscribeListener();
        return true;
    }

    @Override
    public void onDestroy() {
        handler.removeMessages(0);
        syrinx.stopListening();
        syrinx.stopLibrary();
    }


    /**
     * Подписывается листенер на сервис.
     *
     * @param listener листенер
     * @return @false, если листенер уже есть
     */
    public boolean subscribe(SyrinxListener listener) {
        if (chatterCallback == null) {
            chatterCallback = listener;
            return true;
        }
        return false;
    }

    /**
     * Отписывает текущий листенер .
     *
     * @return @true, если листенер != @null
     */
    public boolean unsubscribe() {
        if (chatterCallback == null) {
            return false;
        } else {
            chatterCallback = null;
            return true;
        }
    }

    void stopListening() {
        syrinx.stopListening();
    }

    /**
     * Отправляет сообщение.
     *
     * @param message сообщение для передачи.
     */
    void transmit(String message) {
        byte[] bytes = message.getBytes();
        if (!(bytes.length < MAX_BYTES_TO_TRANSMIT)) {
            byte[] bytesToTransmit = Arrays.copyOfRange(bytes, 0, MAX_BYTES_TO_TRANSMIT + 1);
            message = new String(bytesToTransmit);
        }

        isTransmittingCancelled = false;
        syrinx.stopListening(); //Don't listen while transmitting
        syrinx.transmit(message);
        syrinx.startListening();

        if (chatterCallback != null) {
            if (isTransmittingCancelled) {
                chatterCallback.onTransmitCanceled(message);
            } else {
                chatterCallback.onTransmitEnded(message);
            }
        }
    }

    /**
     * Останавливает передачу сообщения.
     */
    void stopTransmitting() {
        isTransmittingCancelled = true;
        syrinx.stopTransmitting();
    }

    /**
     * Проверяет передается ли в данный момент сообщение.
     *
     * @return @true если в данный момент идет передача сообщения, @false в противном случае.
     */
    boolean isTransmitting() {
        return syrinx.isTransmitting();
    }
}
